import json
import logging
from path import path
from tornado.websocket import WebSocketHandler
import tornado.ioloop
import tornado.web

# http://bibhas.in/blog/using-jinja2-as-the-template-engine-for-tornado-web-framework/


here = path(__file__).parent
pkg = here.parent


class MainHandler(tornado.web.RequestHandler):
    html = here / 'index.html'
    tvars = dict(react="react-0.11.2",
                 semantic='semantic/packaged')

    @property
    def html_txt(self):
        return self.html.text().format(**self.tvars)

    def get(self):
        self.write(self.html_txt)


class DashboardIO(WebSocketHandler):
    connections = set()
    listeners = {}

    def open(self):
        DashboardIO.connections.add(self)

    def on_message(self, message):
        """
        Dispatch an incoming WebSocket message to the appropriate
        registered listeners.
        """
        data = json.loads(message)
        type_listeners = self.listeners.get(data['message_type'], [])
        any_listeners = self.listeners.get(None, [])
        for listener in type_listeners + any_listeners:
            listener(data)

    def on_close(self):
        DashboardIO.connections.discard(self)

    @classmethod
    def send(cls, message):
        """
        Send a message out over all connected WebSockets.
        """
        for connection in cls.connections:
            try:
                connection.write_message(message)
            except:
                logging.error('Error sending message', exc_info=True)

    @classmethod
    def add_listener(cls, message_type, listener):
        """
        Register a listener for WebSocket messages of `message_type`, which
        can be `None` to listen for all messages, regardless of type.
        """
        cls.listeners.setdefault(message_type, []).append(listener)

static_resources = here / 'static'

application = tornado.web.Application([
    (r'/static/(.*)', tornado.web.StaticFileHandler,
     {'path': static_resources}),
    (r"/dashboard.ws", DashboardIO),
    (r"/", MainHandler),
])


if __name__ == "__main__":
    application.listen(8888)
    tornado.ioloop.IOLoop.instance().start()
