import logging
import unittest
import mock

from reconciler import config
from reconciler import tactics
from reconciler import strategy

# Set the log level high here to avoid messages from deployer in the test
# output. They are an artifact of partial specification for testing.
logging.basicConfig(level=logging.CRITICAL)


class TestStrategy(unittest.TestCase):
    def test_strategy_find_next_tactic(self):
        s = strategy.Strategy(None)
        t1 = tactics.Tactic()
        t1.name = "One"
        t2 = tactics.Tactic()
        t2.name = "Two"
        s.extend([t1, t2])

        self.assertEqual(s.find_next_tactic(), t1)
        t1.state = config.COMPLETE
        self.assertEqual(s.find_next_tactic(), t2)

        t1.state = config.FAILED
        self.assertEqual(s.find_next_tactic(), None)

    def test_runnable(self):
        s = strategy.Strategy(None)
        self.assertFalse(s.runnable)
        s.append(tactics.Tactic())
        self.assertTrue(s.runnable)

    @mock.patch.object(strategy.StateEngine, 'env')
    @mock.patch.object(strategy, 'Service')
    def test_sort_by_placement(self, Service, env):
        s = strategy.StateEngine(None)
        s._expected = {'services': {
            's1': 's1',
            's2': 's2',
            's3': 's3',
            's4': 's4',
        }}
        Service.side_effect = lambda n, s: {
            's1': mock.Mock(unit_placement=['kvm:s3=1', 'kvm:s4']),
            's2': mock.Mock(unit_placement=['s3=1', 's1']),
            's3': mock.Mock(unit_placement=[]),
            's4': mock.Mock(unit_placement=['s3']),
        }[n]
        expected = ['s3', 's1', 's2']
        actual = s.sort_by_placement(['s1', 's2', 's3'])
        self.assertEqual(actual, expected)
